@@ -96,6 +96,8 @@ my %ctboID = (
     "\xbe\x7a\xcf\xcb\x97\xa9\x42\xe8\x9c\x71\x99\x94\x91\xe3\xaf\xac" => 1, # XMP
     "\xea\xf4\x2b\x5e\x1c\x98\x4b\x88\xb9\xfb\xb7\xdc\x40\x6e\x4d\x16" => 2, # PreviewImage
     # ID 3 is used for 'mdat' atom (not a uuid)
+    # (haven't seen ID 4 yet)
+    "\x57\x66\xb8\x29\xbb\x6a\x47\xc5\xbc\xfb\x8b\x9f\x22\x60\xd0\x6d" => 5, # something to do with burst-roll image
 );
 
 # mark UserData tags that don't have ItemList counterparts as Preferred
@@ -167,8 +169,11 @@ sub ConvInvISO6709($)
         #  with more than 5 digits after the decimal place:
         #  https://exiftool.org/forum/index.php?topic=11055.msg67171#msg67171 )
         my @fmt = ('%s%02d.%s%s','%s%03d.%s%s','%s%d.%s%s');
+        my @limit = (90,180);
         foreach (@a) {
             return undef unless Image::ExifTool::IsFloat($_);
+            my $lim = shift @limit;
+            warn((@limit ? 'Lat' : 'Long') . "itude out of range\n") if $lim and abs($_) > $lim;
             $_ =~ s/^([-+]?)(\d+)\.?(\d*)/sprintf(shift(@fmt),$1||'+',$2,$3,length($3)<3 ? '0'x(3-length($3)) : '')/e;
         }
         return join '', @a, '/';
@@ -306,14 +311,17 @@ sub CheckQTValue($$$)
 
 #------------------------------------------------------------------------------
 # Format QuickTime value for writing
-# Inputs: 0) ExifTool ref, 1) value ref, 2) Format (or undef), 3) Writable (or undef)
-# Returns: Flags for QT data type, and reformats value as required
+# Inputs: 0) ExifTool ref, 1) value ref, 2) tagInfo ref, 3) Format (or undef)
+# Returns: Flags for QT data type, and reformats value as required (sets to undef on error)
 sub FormatQTValue($$;$$)
 {
-    my ($et, $valPt, $format, $writable) = @_;
+    my ($et, $valPt, $tagInfo, $format) = @_;
+    my $writable = $$tagInfo{Writable};
+    my $count = $$tagInfo{Count};
     my $flags;
+    $format or $format = $$tagInfo{Format};
     if ($format and $format ne 'string' or not $format and $writable and $writable ne 'string') {
-        $$valPt = WriteValue($$valPt, $format || $writable);
+        $$valPt = WriteValue($$valPt, $format || $writable, $count);
         if ($writable and $qtFormat{$writable}) {
             $flags = $qtFormat{$writable};
         } else {
@@ -329,6 +337,7 @@ sub FormatQTValue($$;$$)
         $flags = 0x01;  # UTF8
         $$valPt = $et->Encode($$valPt, 'UTF8');
     }
+    defined $$valPt or $et->WarnOnce("Error converting value for $$tagInfo{Name}");
     return $flags;
 }
 
@@ -347,6 +356,18 @@ sub SetVarInt($$)
     return '';
 }
 
+#------------------------------------------------------------------------------
+# Write Nextbase infi atom (ref PH)
+# Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
+# Returns: updated infi data
+sub WriteNextbase($$$)
+{
+    my ($et, $dirInfo, $tagTablePtr) = @_;
+    $et or return 1;
+    $$et{DEL_GROUP}{Nextbase} and ++$$et{CHANGED}, return '';
+    return ${$$dirInfo{DataPt}};
+}
+
 #------------------------------------------------------------------------------
 # Write Meta Keys to add/delete entries as necessary ('mdta' handler) (ref PH)
 # Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
@@ -776,7 +797,7 @@ sub WriteQuickTime($$$)
     my ($rtnVal, $rtnErr) = $dataPt ? (undef, undef) : (1, 0);
 
     if ($dataPt) {
-        $raf = new File::RandomAccess($dataPt);
+        $raf = File::RandomAccess->new($dataPt);
     } else {
         return 0 unless $raf;
     }
@@ -969,16 +990,14 @@ sub WriteQuickTime($$$)
                 }
             } elsif ($tag eq 'CTBO' or $tag eq 'uuid') { # hack for updating CR3 CTBO offsets
                 push @{$$dirInfo{ChunkOffset}}, [ $tag, length($$outfile), length($hdr) + $size ];
-            } elsif (not $flg) {
-                my $grp = $$et{CUR_WRITE_GROUP} || $parent;
-                $et->Error("Can't locate data reference to update offsets for $grp");
-                return $rtnVal;
+            } elsif (not $flg or $flg == 1) {
+                # assume "1" if stsd is yet to be read
+                $flg or $$et{AssumedDataRef} = 1;
+                # must update offsets since the data is in this file
+                push @{$$dirInfo{ChunkOffset}}, [ $tag, length($$outfile) + length($hdr), $size ];
             } elsif ($flg == 3) {
                 $et->Error("Can't write files with mixed internal/external media data");
                 return $rtnVal;
-            } elsif ($flg == 1) {
-                # must update offsets since the data is in this file
-                push @{$$dirInfo{ChunkOffset}}, [ $tag, length($$outfile) + length($hdr), $size ];
             }
         }
 
@@ -1032,8 +1051,10 @@ sub WriteQuickTime($$$)
 
             if ($subdir) {  # process atoms in this container from a buffer in memory
 
-                undef $$et{HandlerType} if $tag eq 'trak';  # init handler type for this track
-
+                if ($tag eq 'trak') {
+                    undef $$et{HandlerType};  # init handler type for this track
+                    delete $$et{AssumedDataRef};
+                }
                 my $subName = $$subdir{DirName} || $$tagInfo{Name};
                 my $start = $$subdir{Start} || 0;
                 my $base = ($$dirInfo{Base} || 0) + $raf->Tell() - $size;
@@ -1050,6 +1071,7 @@ sub WriteQuickTime($$$)
                     Parent   => $dirName,
                     DirName  => $subName,
                     Name     => $$tagInfo{Name},
+                    TagInfo  => $tagInfo,
                     DirID    => $tag,
                     DataPt   => \$buff,
                     DataLen  => $size,
@@ -1099,6 +1121,11 @@ sub WriteQuickTime($$$)
                     $$et{CHANGED} = $oldChanged;
                     undef $newData;
                 }
+                if ($tag eq 'trak' and $$et{AssumedDataRef}) {
+                    my $grp = $$et{CUR_WRITE_GROUP} || $dirName;
+                    $et->Error("Can't locate data reference to update offsets for $grp");
+                    delete $$et{AssumedDataRef};
+                }
                 $$et{CUR_WRITE_GROUP} = $oldWriteGroup;
                 SetByteOrder('MM');
                 # add back header if necessary
@@ -1139,7 +1166,7 @@ sub WriteQuickTime($$$)
                                         my $newVal = $et->GetNewValue($nvHash);
                                         next unless defined $newVal;
                                         my $prVal = $newVal;
-                                        my $flags = FormatQTValue($et, \$newVal, $format, $$tagInfo{Writable});
+                                        my $flags = FormatQTValue($et, \$newVal, $tagInfo, $format);
                                         next unless defined $newVal;
                                         my ($ctry, $lang) = (0, 0);
                                         if ($$ti{LangCode}) {
@@ -1220,7 +1247,8 @@ sub WriteQuickTime($$$)
                                     }
                                     my $prVal = $newVal;
                                     # format new value for writing (and get new flags)
-                                    $flags = FormatQTValue($et, \$newVal, $format, $$tagInfo{Writable});
+                                    $flags = FormatQTValue($et, \$newVal, $tagInfo, $format);
+                                    next unless defined $newVal;
                                     my $grp = $et->GetGroup($langInfo, 1);
                                     $et->VerboseValue("- $grp:$$langInfo{Name}", $val);
                                     $et->VerboseValue("+ $grp:$$langInfo{Name}", $prVal);
@@ -1400,6 +1428,13 @@ sub WriteQuickTime($$$)
                 $flg = 1; # (this seems to be the case)
             }
             $$et{QtDataFlg} = $flg;
+            if ($$et{AssumedDataRef}) {
+                if ($flg != $$et{AssumedDataRef}) {
+                    my $grp = $$et{CUR_WRITE_GROUP} || $parent;
+                    $et->Error("Assumed incorrect data reference for $grp (was $flg)");
+                }
+                delete $$et{AssumedDataRef};
+            }
         }
         if ($tagInfo and $$tagInfo{WriteLast}) {
             $writeLast = ($writeLast || '') . $hdr . $buff;
@@ -1463,7 +1498,7 @@ sub WriteQuickTime($$$)
                 my $newVal = $et->GetNewValue($nvHash);
                 next unless defined $newVal;
                 my $prVal = $newVal;
-                my $flags = FormatQTValue($et, \$newVal, $$tagInfo{Format}, $$tagInfo{Writable});
+                my $flags = FormatQTValue($et, \$newVal, $tagInfo);
                 next unless defined $newVal;
                 my ($ctry, $lang) = (0, 0);
                 # handle alternate languages
@@ -1596,7 +1631,8 @@ sub WriteQuickTime($$$)
         if ($isEmpty) {
             $et->VPrint(0,'  Deleting ' . join('+', sort map { $emptyMeta{$_} } keys %boxPos)) if %boxPos;
             $$outfile = '';
-            ++$$et{CHANGED};
+            # (could report a file if editing nothing when it contained an empty Meta atom)
+            # ++$$et{CHANGED};
         }
         if ($curPath eq 'MOV-Movie-Meta') {
             delete $$addDirs{Keys}; # prevent creation of another Meta for Keys tags
@@ -1662,12 +1698,18 @@ sub WriteQuickTime($$$)
                 # edit size of mdat in header if necessary
                 if ($diff) {
                     if (length($$hdrChunk[2]) == 8) {
-                        my $size = Get32u(\$$hdrChunk[2], 0) + $diff;
-                        $size > 0xffffffff and $et->Error("Can't yet grow mdat across 4GB boundary"), return $rtnVal;
-                        Set32u($size, \$$hdrChunk[2], 0);
+                        my $size = Get32u(\$$hdrChunk[2], 0);
+                        if ($size) { # (0 size = extends to end of file)
+                            $size += $diff;
+                            $size > 0xffffffff and $et->Error("Can't yet grow mdat across 4GB boundary"), return $rtnVal;
+                            Set32u($size, \$$hdrChunk[2], 0);
+                        }
                     } elsif (length($$hdrChunk[2]) == 16) {
-                        my $size = Get64u(\$$hdrChunk[2], 8) + $diff;
-                        Set64u($size, \$$hdrChunk[2], 8);
+                        my $size = Get64u(\$$hdrChunk[2], 8);
+                        if ($size) {
+                            $size += $diff;
+                            Set64u($size, \$$hdrChunk[2], 8);
+                        }
                     } else {
                         $et->Error('Internal error. Invalid mdat header');
                         return $rtnVal;
@@ -1894,7 +1936,12 @@ sub WriteMOV($$)
         $ftype = 'MOV';
     }
     $et->SetFileType($ftype); # need to set "FileType" tag for a Condition
-    $et->InitWriteDirs($dirMap{$ftype}, 'XMP', 'QuickTime');
+    if ($ftype eq 'HEIC') {
+        # EXIF is preferred in HEIC files
+        $et->InitWriteDirs($dirMap{$ftype}, 'EXIF', 'QuickTime');
+    } else {
+        $et->InitWriteDirs($dirMap{$ftype}, 'XMP', 'QuickTime');
+    }
     $$et{DirMap} = $dirMap{$ftype};     # need access to directory map when writing
     # track tags globally to avoid creating multiple tags in the case of duplicate directories
     $$et{DidTag} = { };
@@ -1927,7 +1974,7 @@ QuickTime-based file formats like MOV and MP4.
 
 =head1 AUTHOR
 
-Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2024, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
